from typing import Optional
from attr import dataclass


@dataclass
class Route:
    id: str
    distance: int
    dst_address: str
    dynamic: bool
    ecmp: bool
    gateway: str
    hw_offloaded: bool
    immediate_gw: str
    inactive: bool
    routing_table: str
    scope: int
    suppress_hw_offload: bool
    active: bool = False
    pref_src: Optional[str] = None
    target_scope: Optional[int] = None
    vpn: Optional[bool] = None
    vrf_interface: Optional[str] = None
    comment: Optional[str] = None



from attr import dataclass
from typing import Optional


@dataclass
class DHCPClient:
    id: str
    add_default_route: str
    dhcp_options: str
    disabled: bool
    dynamic: bool
    interface: str
    invalid: bool
    use_peer_dns: bool
    use_peer_ntp: bool
    address: Optional[str] = None
    comment: Optional[str] = None
    default_route_distance: Optional[int] = None
    dhcp_server: Optional[str] = None
    expires_after: Optional[str] = None
    gateway: Optional[str] = None
    primary_dns: Optional[str] = None
    primary_ntp: Optional[str] = None
    secondary_dns: Optional[str] = None
    status: Optional[str] = None

    def __str__(self) -> str:
        return self.interface



from attr import dataclass


@dataclass
class DHCPRelay:
    id: str
    add_relay_info: bool
    delay_threshold: str
    dhcp_server: str
    disabled: bool
    interface: str
    invalid: bool
    local_address: str
    name: str

    def __str__(self) -> str:
        return self.name



from attr import define
from typing import Any, List

from ros._base import BaseSubModule

from .ethernet import InterfaceEthernet


@define
class EthernetModule(BaseSubModule):
    def __call__(self, **kwds: Any) -> List[InterfaceEthernet]:
        return self.print(**kwds)

    def print(self, **kwds) -> List[InterfaceEthernet]:
        return self.module.ros.get_as(self.url, List[InterfaceEthernet], kwds)


__all__ = ["InterfaceEthernet", "EthernetModule"]



from attr import dataclass
from typing import Optional


@dataclass
class Bridge:
    id: str
    ageing_time: str
    arp: str
    arp_timeout: str
    auto_mac: bool
    dhcp_snooping: bool
    disabled: bool
    fast_forward: bool
    forward_delay: str
    igmp_snooping: bool
    mac_address: str
    max_message_age: str
    mtu: str
    name: str
    priority: str
    protocol_mode: str
    running: bool
    transmit_hold_count: int
    vlan_filtering: bool
    actual_mtu: Optional[int] = None
    l2mtu: Optional[int] = None
    comment: Optional[str] = None

    def __str__(self) -> str:
        return self.name



from attr import dataclass
from typing import Optional


@dataclass
class DHCPServer:
    id: str
    address_pool: str
    authoritative: bool
    disabled: bool
    dynamic: bool
    interface: str
    invalid: bool
    lease_script: str
    lease_time: str
    name: str
    use_radius: str
    add_arp: Optional[bool] = None
    comment: Optional[str] = None
    insert_queue_before: Optional[str] = None
    parent_queue: Optional[str] = None

    def __str__(self) -> str:
        return self.name



from attr import define
from typing import TYPE_CHECKING, Any, Generic, List, Type, TypeVar, Union

if TYPE_CHECKING:
    from ros.ros import Ros


@define
class BaseModule:
    ros: "Ros"
    url: str = ""

    def __attrs_post_init__(self) -> None:
        if not self.url:
            cname = self.__class__.__name__.lower()
            self.url = "/" + cname.replace("module", "")


BM = TypeVar("BM", bound=BaseModule)


@define
class BaseSubModule:
    module: BM
    url: str = ""
    _ros: "Ros" = None

    @property
    def ros(self) -> "Ros":
        if not self._ros:
            self._ros = self.module.ros
        return self._ros

    def __attrs_post_init__(self) -> None:
        if self.url:
            if not self.url.startswith(self.module.url):
                self.url = self.module.url + self.url
        else:
            cname = self.__class__.__name__.lower()
            self.url = self.module.url + "/" + cname.replace("module", "")


PR = TypeVar("PR", bound=object)


@define
class BaseProp(Generic[PR]):
    mod: Union[BaseModule, BaseSubModule]
    url: str
    cl: Type[PR]

    def __call__(self, **kwds: Any) -> PR:
        return self.print(**kwds)

    def print(self, **kwds: Any) -> PR:
        return self.mod.ros.get_as(self.url, self.cl, kwds)


@define
class BaseProps(Generic[PR]):
    mod: Union[BaseModule, BaseSubModule]
    url: str
    cl: Type[PR]

    def __call__(self, **kwds: Any) -> List[PR]:
        return self.print(**kwds)

    def print(self, **kwds: Any) -> List[PR]:
        return self.mod.ros.get_as(self.url, List[self.cl], kwds)



from typing import List
from ros._base import BaseSubModule

from ._literals import UpdateChannel
from .package import Package
from .update import Update, UpdateModule


class PackageModule(BaseSubModule):
    _update: UpdateModule = None

    def downgrade(self):
        self.module.ros.post_as(self.url + "/downgrade", None)

    def __call__(self, **kwds) -> List[Package]:
        return self.print(**kwds)

    def print(self, **kwds) -> List[Package]:
        return self.module.ros.get_as(self.url, List[Package], kwds)

    @property
    def update(self) -> UpdateModule:
        if not self._update:
            self._update = UpdateModule(self.module, "/package/update")
        return self._update


__all__ = ["Package", "PackageModule", "Update", "UpdateChannel", "UpdateModule"]



from attrs import define
from typing import Any, List

from ros._base import BaseSubModule

from .bridge import Bridge
from .msti import BridgeMsti
from .port import BridgePort
from .vlan import BridgeVlan


@define
class BridgeModule(BaseSubModule):
    def __call__(self, **kwds: Any) -> List[Bridge]:
        return self.print(**kwds)

    def print(self, **kwds: Any) -> List[Bridge]:
        return self.module.ros.get_as(self.url, List[Bridge], kwds)

    @property
    def msti(self) -> List[BridgeMsti]:
        return self.module.ros.get_as(self.url + "/msti", List[BridgeMsti])

    @property
    def port(self) -> List[BridgePort]:
        return self.module.ros.get_as(self.url + "/port", List[BridgePort])

    @property
    def vlan(self) -> List[BridgeVlan]:
        return self.module.ros.get_as(self.url + "/vlan", List[BridgeVlan])


__all__ = ["BridgeModule", "BridgeInterface", "BridgePort"]



from typing import Any, List

from ros._base import BaseSubModule
from .connection import IPConnection


class IPFirewallConnectionModule(BaseSubModule):
    def __call__(self, **kwds: Any) -> List[IPConnection]:
        return self.print(**kwds)

    def print(self, **kwds: Any) -> List[IPConnection]:
        return self.module.ros.get_as(self.url, List[IPConnection], kwds)



from attr import dataclass
from typing import Optional


@dataclass
class ARP:
    id: str
    DHCP: bool
    address: str
    complete: bool
    disabled: bool
    dynamic: bool
    interface: str
    invalid: bool
    published: bool
    mac_address: Optional[str] = None
    comment: Optional[str] = None



from attr import define
from typing import Any, List

from ros._base import BaseSubModule

from .list import InterfaceList
from .member import InterfaceListMember


@define
class InterfaceListModule(BaseSubModule):
    def __call__(self, **kwds: Any) -> Any:
        return self.print(**kwds)

    def print(self, **kwds) -> List[InterfaceList]:
        return self.module.ros.get_as(self.url, List[InterfaceList], kwds)

    def member(self, **kwds) -> List[InterfaceListMember]:
        return self.module.ros.get_as(
            self.url + "/member", List[InterfaceListMember], kwds
        )


__all__ = ["InterfaceList", "InterfaceListMember", "InterfaceListModule"]



from attr import dataclass
from typing import Optional


@dataclass
class Address:
    id: str
    actual_interface: str
    address: str
    disabled: bool
    dynamic: bool
    interface: str
    invalid: bool
    network: str
    comment: Optional[str] = None



from typing import Optional

from ros._base import BaseSubModule

from .connection import IPFirewallConnectionModule
from .filter import IPFirewallFilterModule
from .layer7_protocol import IPFirewallLayer7ProtocolModule
from .mangle import IPFirewallMangleModule
from .nat import IPFirewallNatModule
from .raw import IPFirewallRawModule
from .service_port import IPFirewallServicePortModule


class IPFirewallModule(BaseSubModule):
    _connection: Optional[IPFirewallConnectionModule] = None
    _filter: Optional[IPFirewallFilterModule] = None
    _layer7_protocol: Optional[IPFirewallLayer7ProtocolModule] = None
    _mangle: Optional[IPFirewallMangleModule] = None
    _nat: Optional[IPFirewallNatModule] = None
    _raw: Optional[IPFirewallRawModule] = None
    _service_port: Optional[IPFirewallServicePortModule] = None

    @property
    def connection(self):
        if not self._connection:
            self._connection = IPFirewallConnectionModule(
                self.module, self.url + "/connection"
            )
        return self._connection

    @property
    def filter(self):
        if not self._filter:
            self._filter = IPFirewallFilterModule(self.module, self.url + "/filter")
        return self._filter

    @property
    def layer7_protocol(self):
        if not self._layer7_protocol:
            self._layer7_protocol = IPFirewallLayer7ProtocolModule(
                self.module, self.url + "/layer7-protocol"
            )
        return self._layer7_protocol

    @property
    def mangle(self):
        if not self._mangle:
            self._mangle = IPFirewallMangleModule(self.module, self.url + "/mangle")
        return self._mangle

    @property
    def nat(self):
        if not self._nat:
            self._nat = IPFirewallNatModule(self.module, self.url + "/nat")
        return self._nat

    @property
    def raw(self):
        if not self._raw:
            self._raw = IPFirewallRawModule(self.module, self.url + "/raw")
        return self._raw

    @property
    def service_port(self):
        if not self._service_port:
            self._service_port = IPFirewallServicePortModule(
                self.module, self.url + "/service-port"
            )
        return self._service_port



from typing import List

from ros._base import BaseSubModule
from .dhcp_server import DHCPServer
from .network import DHCPNetwork


class DHCPServerModule(BaseSubModule):
    url: str = "dhcp-server"

    def __call__(self, **kwds) -> List[DHCPServer]:
        return self.print(**kwds)

    def print(self, **kwds) -> List[DHCPServer]:
        return self.module.ros.get_as(self.url, List[DHCPServer], kwds)

    @property
    def network(self) -> List[DHCPNetwork]:
        return self.module.ros.get_as(self.url + "/network", List[DHCPNetwork])


__all__ = ["DHCPNetwork", "DHCPServerModule", "DHCPServer"]



from attr import dataclass
from typing import Optional


@dataclass
class DHCPNetwork:
    id: str
    address: str
    caps_manager: str
    dhcp_option: str
    dns_server: str
    dynamic: bool
    gateway: str
    ntp_server: str
    wins_server: str
    comment: Optional[str] = None



from attr import dataclass
from typing import List, Optional

from ros._base import BaseSubModule
from . import UpdateChannel


@dataclass
class Update:
    channel: UpdateChannel
    installed_version: str
    latest_version: Optional[str] = None
    status: Optional[str] = None
    section: Optional[int] = None


class UpdateModule(BaseSubModule):
    def print(self) -> Update:
        return self.module.ros.get_as(self.url, Update)

    def cancel(self):
        self.module.ros.post_as(self.url + "/cancel", None)

    def check_for_updates(self) -> Update:
        return self.module.ros.post_as(self.url + "/check-for-updates", List[Update])

    def download(self):
        self.module.ros.post_as(self.url + "/download", None)

    def install(self):
        self.module.ros.post_as(self.url + "/install", None)

    def set(self, channel: UpdateChannel):
        data = {"channel": channel}
        return self.module.ros.post_as(self.url + "/set", None, data)
